@@ -1,11 +1,13 @@
 ;;; iedit-tests.el --- iedit's automatic-tests
 
-;; Copyright (C) 2010, 2011, 2012 Victor Ren
+;; Copyright (C) 2010 - 2019, 2020 Victor Ren
 
-;; Time-stamp: <2016-09-20 00:28:29 Victor Ren>
+;; Time-stamp: <2022-01-14 12:33:56 Victor Ren>
 ;; Author: Victor Ren <victorhge@gmail.com>
-;; Version: 0.97
-;; X-URL: http://www.emacswiki.org/emacs/Iedit
+;; Version: 0.9.9.9.9
+;; X-URL: https://github.com/victorhge/iedit
+;;        https://www.emacswiki.org/emacs/Iedit
+;; Compatibility: GNU Emacs: 22.x, 23.x, 24.x, 25.x
 
 ;; This file is not part of GNU Emacs, but it is distributed under
 ;; the same terms as GNU Emacs.
@@ -31,11 +33,18 @@
 (require 'ert)
 (require 'iedit)
 (require 'iedit-rect)
+(require 'elp)
 
-(ert-deftest iedit-compile-test ()
-  (let ((byte-compile-error-on-warn t ))
-    (should (byte-compile-file "~/.emacs.d/site-lisp/iedit/iedit.el"))
-    (delete-file "~/.emacs.d/site-lisp/iedit/iedit.elc" nil)))
+(ert-deftest iedit-batch-compile-test ()
+  (with-temp-buffer
+	(call-process-shell-command "emacs -L . -Q --batch -f batch-byte-compile *.el" nil (current-buffer))
+    (should (string= (buffer-string) "Iedit default key binding is C-;
+Iedit-rect default key binding is <C-x> <r> <;>
+"))
+	(delete-file (byte-compile-dest-file "iedit-lib.el") nil)
+	(delete-file (byte-compile-dest-file "iedit-rect.el") nil)
+	(delete-file (byte-compile-dest-file "iedit-tests.el") nil)
+	(delete-file (byte-compile-dest-file "iedit.el") nil)))
 
 (defmacro with-iedit-test-buffer (buffer-name &rest body)
   (declare (indent 1) (debug t))
@@ -61,7 +70,6 @@
 (defun goto-word-beginning (word)
   (goto-word word t))
 
-
 (defun with-iedit-test-fixture (input-buffer-string body)
   "iedit test fixture"
   (let ((old-transient-mark-mode transient-mark-mode)
@@ -71,12 +79,14 @@
           (with-iedit-test-buffer "* iedit transient mark *"
             (transient-mark-mode t)
             (setq iedit-transient-mark-sensitive t)
+			(setq iedit-case-sensitive t)
             (insert input-buffer-string)
             (goto-char 1)
             (iedit-mode)
             (funcall body))
           (with-iedit-test-buffer "* iedit NO transient mark *"
             (setq iedit-transient-mark-sensitive nil)
+			(setq iedit-case-sensitive t)
             (transient-mark-mode -1)
             (insert input-buffer-string)
             (goto-char 1)
@@ -165,7 +175,8 @@ foo"
      (should (string= iedit-initial-string-local "foo"))
      (should (eq 'selection iedit-occurrence-type-local))
      (goto-char 1)
-     (insert "123")
+	 (insert "123")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "123foobar
  123foo123foo123foo
@@ -173,6 +184,7 @@ foo"
  123foo"))
      (forward-char 3)
      (insert "456")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "123foo456bar
  123foo456123foo456123foo456
@@ -185,39 +197,77 @@ foo"
 foo"
    (lambda ()
      (iedit-mode)
-     (highlight-changes-mode 1)
      (goto-char (point-min))
      (goto-char (point-at-eol))
      (iedit-mode)
      (delete-region (point) (1- (point)))
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
                       "fo
 fo"))
      (insert "b")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
                       "fob
 fob")))))
 
 (ert-deftest iedit-mode-start-from-isearch-test ()
   (with-iedit-test-fixture
+"a
+(defun foo (foo bar foo)
+\"foo bar foobar\" nil)
+ (defun bar (bar foo bar)
+  \"bar foo barfoo\" nil)
+foo
+ foo"
+   (lambda ()
+      (iedit-mode)
+      (emacs-lisp-mode)
+      (goto-char 5)
+      (iedit-mode)
+      (isearch-mode t)
+      (isearch-process-search-char ?f)
+      (isearch-process-search-char ?o)
+      (isearch-process-search-char ?o)
+      (iedit-mode-from-isearch 0)
+      (should (string= iedit-initial-string-local "foo"))
+      (should (= 5 (length iedit-occurrences-overlays)))
+	  (iedit-mode)
+	  (isearch-mode t)
+      (isearch-process-search-char ?f)
+      (isearch-process-search-char ?o)
+      (isearch-process-search-char ?o)
+      (iedit-mode-from-isearch)
+	  (should (= 10 (length iedit-occurrences-overlays)))
+	  )))
+
+(ert-deftest iedit-mode-start-from-isearch-regexp-test ()
+  (with-iedit-test-fixture
 "foo
-  foo
+  fobar
+  foobar
+  fooobar
    barfoo
    foo"
    (lambda ()
-     (should (= 3 (length iedit-occurrences-overlays)))
-     (should (string= iedit-initial-string-local "foo"))
      (iedit-mode)
-     (forward-line 2)
-     (isearch-mode t)
+     (call-interactively 'isearch-forward-regexp)
      (isearch-process-search-char ?f)
      (isearch-process-search-char ?o)
-     (isearch-process-search-char ?o)
+     (isearch-process-search-char ?*)
+     (isearch-process-search-char ?b)
      (call-interactively 'iedit-mode-from-isearch)
-     (should (string= iedit-initial-string-local "foo"))
-     (should (= 4 (length iedit-occurrences-overlays)))
-     (iedit-mode)
-     (should (null iedit-occurrences-overlays)))))
+     (should (null iedit-occurrences-overlays))
+     (should (null iedit-mode)) ;; (should (string= (current-message) "Matches are not the same length."))
+     (goto-char (point-min))
+     (call-interactively 'isearch-forward-regexp)
+     (isearch-process-search-char ?f)
+     (isearch-process-search-char ?o)
+     (isearch-process-search-char ?.)
+     (isearch-process-search-char ?b)
+     (call-interactively 'iedit-mode-from-isearch)
+     (should (= 1 (length iedit-occurrences-overlays)))
+    )))
 
 (ert-deftest iedit-mode-last-local-occurrence-test ()
   (with-iedit-test-fixture
@@ -275,30 +325,36 @@ fob")))))
    (lambda ()
      (iedit-goto-last-occurrence)
      (should (= (point) 24))
+     (should (= iedit-occurrence-index 3))
      (iedit-goto-first-occurrence)
      (should (= (point) 1))
-     (iedit-next-occurrence)
+     (should (= iedit-occurrence-index 1))
+     (iedit-next-occurrence 1)
      (should (= (point) 7))
-     (iedit-next-occurrence)
+     (should (= iedit-occurrence-index 2))
+     (iedit-next-occurrence 1)
      (should (= (point) 24))
-     (iedit-next-occurrence)
+     (should (= iedit-occurrence-index 3))
+     (iedit-next-occurrence 1)
      (should (= (point) 24)) ;; (should (string= (current-message) "This is the last occurrence."))
-     (iedit-next-occurrence)
+     (should (= iedit-occurrence-index 3))
+     (iedit-next-occurrence 1)
      (should (= (point) 1)) ;; (should (string= (current-message) "Located the first occurrence."))
-     (iedit-next-occurrence)
+     (should (= iedit-occurrence-index 1))
+     (iedit-next-occurrence 1)
      (should (= (point) 7))
+     (should (= iedit-occurrence-index 2))
      (goto-char (point-max))
-     (iedit-prev-occurrence)
-     (should (= (point) 27))
-     (iedit-prev-occurrence)
+     (iedit-prev-occurrence 1)
      (should (= (point) 24))
-     (iedit-prev-occurrence)
+     (should (= iedit-occurrence-index 3))
+     (iedit-prev-occurrence 1)
      (should (= (point) 7))
-     (iedit-prev-occurrence)
+     (iedit-prev-occurrence 1)
      (should (= (point) 1))
-     (iedit-prev-occurrence)
+     (iedit-prev-occurrence 1)
      (should (= (point) 1)) ;; (should (string= (current-message) "This is the first occurrence."))
-     (iedit-prev-occurrence)
+     (iedit-prev-occurrence 1)
      (should (= (point) 24)) ;; (should (string= (current-message) "Located the last occurrence."))
      )))
 
@@ -310,18 +366,21 @@ fob")))))
    foo"
    (lambda ()
      (insert "1")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "1foo
   1foo
    barfoo
    1foo"))
      (backward-delete-char 1)
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "foo
   foo
    barfoo
    foo"))
      (capitalize-word 1)
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "Foo
   Foo
@@ -330,6 +389,7 @@ fob")))))
      ;; test insert from empty
      (iedit-delete-occurrences)
      (insert "1")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "1
   1
@@ -355,6 +415,7 @@ fob")))))
    foo"))
      (goto-char 7)
      (insert "1")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "foo
   1foo
@@ -385,10 +446,57 @@ fob")))))
      (should (= 2 (length iedit-occurrences-overlays)))
      (iedit-toggle-case-sensitive)
      (should (= 3 (length iedit-occurrences-overlays)))
-     (iedit-next-occurrence)
+     (iedit-next-occurrence 1)
      (iedit-toggle-case-sensitive)
      (should (= 1 (length iedit-occurrences-overlays))))))
 
+(ert-deftest iedit-toggle-search-invisible-test ()
+  (with-iedit-test-fixture
+"foo
+* foo
+** foo"
+   (lambda ()
+     (iedit-mode) ; turn off iedit-mode
+	 (outline-mode)
+	 (forward-line 1)
+	 (call-interactively 'outline-hide-subtree)
+	 (setq iedit-search-invisible t)
+	 (goto-char 1)
+	 (iedit-mode)
+     (should (= 3 (length iedit-occurrences-overlays)))
+	 (should (= 0 iedit-lib-skip-invisible-count))
+     (iedit-toggle-search-invisible)
+     (should (= 2 (length iedit-occurrences-overlays)))
+	 (should (null iedit-search-invisible))
+	 (should (= 1 iedit-lib-skip-invisible-count))
+	 (iedit-toggle-search-invisible)
+	 (should (= 3 (length iedit-occurrences-overlays)))
+	 (should (= 0 iedit-lib-skip-invisible-count))
+	 (should (eq 'open iedit-search-invisible)))))
+
+(ert-deftest iedit-case-preserve-test ()
+  (with-iedit-test-fixture
+"foo
+  Foo
+   barFoo
+   FOO"
+(lambda ()
+  (iedit-mode)
+  (goto-char 1)
+  (set-mark-command nil)
+  (forward-char 3)
+  (let ((iedit-case-sensitive nil)
+		(case-replace t))
+	(iedit-mode)
+	(goto-char 1)
+	(insert "bar")
+	(run-hooks 'post-command-hook)
+	(should (string= (buffer-string)
+"barfoo
+  BarFoo
+   barBarFoo
+   BARFOO"))))))
+
 (ert-deftest iedit-apply-on-occurrences-test ()
   "Test functions deal with the whole occurrences"
   (with-iedit-test-fixture
@@ -415,7 +523,7 @@ fob")))))
   bar
    barfoo
    bar"))
-     (iedit-number-occurrences 1)
+     (iedit-number-occurrences 1 "%d ")
      (should (string= (buffer-string)
 "1 bar
   2 bar
@@ -462,6 +570,7 @@ fob")))))
    (lambda ()
      (iedit-toggle-buffering)
      (insert "bar")
+	 (run-hooks 'post-command-hook)
      (should (string= (buffer-string)
 "barfoo
  foo
@@ -486,9 +595,79 @@ fob")))))
      (iedit-toggle-buffering)
      (should (string= (buffer-string)
 "foo
+ foo
+  barfoo
+    foo")))))
+
+(ert-deftest iedit-buffering-undo-test ()
+  (with-iedit-test-fixture
+"foo
+ foo
+  barfoo
+    foo"
+   (lambda ()
+	 (iedit-mode)						;turnoff
+     (setq iedit-auto-buffering t)
+	 (push nil buffer-undo-list)
+	 (call-interactively 'iedit-mode)
+     (insert "bar")
+	 (run-hooks 'post-command-hook)
+     (should (string= (buffer-string)
+"barfoo
+ foo
+  barfoo
+    foo"))
+     (call-interactively 'iedit-mode)
+     (should (string= (buffer-string)
+"barfoo
  barfoo
   barfoo
-    barfoo")))))
+    barfoo"))
+     (should (= (point) 4))
+	 (push nil buffer-undo-list)
+	 (undo 1)
+	 (should (= (point) 1))
+     (should (string= (buffer-string)
+"foo
+ foo
+  barfoo
+    foo"))
+     )))
+
+(ert-deftest iedit-buffering-quit-test ()
+  (with-iedit-test-fixture
+"foo
+ foo
+  barfoo
+    foo"
+   (lambda ()
+	 (iedit-mode)						;turnoff
+     (setq iedit-auto-buffering t)
+	 (push nil buffer-undo-list)
+	 (call-interactively 'iedit-mode)
+     (insert "bar")
+	 (run-hooks 'post-command-hook)
+     (should (string= (buffer-string)
+"barfoo
+ foo
+  barfoo
+    foo"))
+     (call-interactively 'iedit--quit)
+     (should (string= (buffer-string)
+"barfoo
+ foo
+  barfoo
+    foo"))
+     (should (= (point) 4))
+	 (push nil buffer-undo-list)
+	 (undo 1)
+	 (should (= (point) 1))
+     (should (string= (buffer-string)
+"foo
+ foo
+  barfoo
+    foo"))
+     )))
 
 (ert-deftest iedit-rectangle-start-test ()
   (with-iedit-test-fixture
@@ -523,7 +702,7 @@ fob")))))
      (call-interactively 'iedit-rectangle-mode)
      (should (iedit-same-column))
      (should (equal (marker-position-list iedit-rectangle) '(1 22)))
-     (iedit-prev-occurrence)
+     (iedit-prev-occurrence 1)
      (delete-char -1)
      (should (not (iedit-same-column)))
      (should-error (iedit-kill-rectangle)))))
@@ -602,11 +781,14 @@ arfoo
       (emacs-lisp-mode)
       (goto-char 5)
       (iedit-mode)
+      (setq iedit-auto-narrow t)
       (iedit-restrict-function)
       (should (= 1 (length iedit-occurrences-overlays)))
+      (should (equal (buffer-narrowed-p) iedit-is-narrowed))
       (iedit-mode)
       (goto-char 13)
-      (iedit-mode-toggle-on-function)
+      (setq iedit-auto-narrow nil)
+      (call-interactively 'iedit-mode-toggle-on-function)
       (should (= 4 (length iedit-occurrences-overlays)))
       (iedit-mode)
       (iedit-mode)
@@ -654,8 +836,8 @@ abcd" "12345678901234567890123456789012345678901234567890...")))
   (dolist (test iedit-printable-test-lists)
     (should (string= (iedit-printable (car test)) (cadr test)))))
 
-(ert-deftest iedit-hide-unmatched-lines-test ()
-  "Test function iedit-hide-unmatched-lines."
+(ert-deftest iedit-hide-context-lines-test ()
+  "Test function iedit-hide-context-lines."
   (with-iedit-test-fixture
    "foo
 foo
@@ -680,13 +862,41 @@ a
 a
 foo"
    (lambda ()
-     (should (equal (iedit-hide-unmatched-lines 0) '((64 73) (47 54) (33 38) (21 24) (9 10))))
+     (should (equal (iedit-hide-context-lines 0) '((64 73) (47 54) (33 38) (21 24) (9 10))))
      (iedit-show-all)
-     (should (equal (iedit-hide-unmatched-lines 1) '((66 71) (49 52) (35 36))))
+     (should (equal (iedit-hide-context-lines 1) '((66 71) (49 52) (35 36))))
      (iedit-show-all)
-     (should (equal (iedit-hide-unmatched-lines 2) '((68 69)) ))
+     (should (equal (iedit-hide-context-lines 2) '((68 69)) ))
      (iedit-show-all)
-     (should (equal (iedit-hide-unmatched-lines 3) nil)))))
+     (should (equal (iedit-hide-context-lines 3) nil)))))
+
+(ert-deftest iedit-hide-occurrence-lines-test ()
+  "Test function iedit-hide-occurrence-lines."
+  (with-iedit-test-fixture
+   "foo
+foo
+a
+  foo bar
+a
+a
+bar foo
+a
+a
+a
+bar foo
+a
+a
+a
+a
+ foo bar
+a
+a
+a
+a
+a
+foo"
+   (lambda ()
+     (should (equal (iedit-hide-occurrence-lines) '((74 77) (55 63) (39 46) (25 32) (11 20) (1 8)))))))
 
 ;; todo add a auto performance test
 (setq elp-function-list '(;; insert-and-inherit
